#include "StdAfx.h"
#include "Loader.h"
 
using namespace mscorlib;
 
std::auto_ptr<CLoader> CLoader::m_pInstance;
 
static HRESULT GetDllDirectory(TCHAR *szPath, DWORD nPathBufferSize);
 
CLoader *CLoader::TheInstance()
{
    if (m_pInstance.get() == NULL)
    {
        m_pInstance = std::auto_ptr<CLoader>(new CLoader());
    }
 
    return m_pInstance.get();
}
 
CLoader::CLoader(void)
    : m_pHost(NULL)
    , m_pLocalDomain(NULL)
{
}
 
CLoader::~CLoader(void)
{
    if (m_pLocalDomain)
    {
        m_pLocalDomain->Release();
    }
 
    if (m_pHost)
    {
        m_pHost->Release();
    }
}
 
HRESULT CLoader::LoadCLR(LPCWSTR szRuntimeVersion)
{
    HRESULT hr = S_OK;
 
    if (m_pHost != NULL)
        return hr;
 
    hr = CorBindToRuntimeEx(
        szRuntimeVersion, 0,
        STARTUP_LOADER_OPTIMIZATION_MULTI_DOMAIN | STARTUP_CONCURRENT_GC,
        CLSID_CorRuntimeHost,
        IID_ICorRuntimeHost,
        (PVOID*) &m_pHost);
 
    if (!SUCCEEDED(hr))
    {
        return hr;
    }
 
    return m_pHost->Start();
}
 
HRESULT CLoader::CreateLocalAppDomain()
{
    USES_CONVERSION;
 
    HRESULT hr = S_OK;
 
    if (m_pLocalDomain != NULL)
    {
        return hr;
    }
 
    CComPtr<IUnknown> pDomainSetupPunk;
    CComPtr<IAppDomainSetup> pDomainSetup;
    CComPtr<IUnknown> pLocalDomainPunk;
    TCHAR szDirectory[MAX_PATH + 1];
 
    IfFailGo( m_pHost->CreateDomainSetup(&pDomainSetupPunk) );
    IfFailGo( pDomainSetupPunk->QueryInterface(__uuidof(pDomainSetup),
        (LPVOID*)&pDomainSetup) );
 
    IfFailGo( ::GetDllDirectory(szDirectory, sizeof(szDirectory)/sizeof(szDirectory[0])) );
 
    pDomainSetup->put_ApplicationBase(CComBSTR(szDirectory));
 
    IfFailGo( m_pHost->CreateDomainEx(T2W(szDirectory),
        pDomainSetupPunk, 0, &pLocalDomainPunk) );
 
    IfFailGo( pLocalDomainPunk->QueryInterface(__uuidof(m_pLocalDomain),
        (LPVOID*)&m_pLocalDomain) );
 
Error:
    return hr;
}
 
HRESULT CLoader::CreateInstance(LPCWSTR szRuntimeVersion, LPCWSTR szAssemblyName, LPCWSTR szClassName, const IID &riid, void ** ppvObject)
{
    HRESULT hr = S_OK;
    _ObjectHandle *pObjHandle = NULL;
    VARIANT v;
 
    IfFailGo( LoadCLR(szRuntimeVersion) );
 
    IfFailGo( CreateLocalAppDomain() );
 
    IfFailGo(m_pLocalDomain->CreateInstance(CComBSTR(szAssemblyName),
        CComBSTR(szClassName), &pObjHandle));
    VariantInit(&v);
    IfFailGo(pObjHandle->Unwrap(&v));
    IfFailGo(v.pdispVal->QueryInterface(riid, ppvObject));
 
Error:
 
    return hr;
}
 
HRESULT CLoader::RegisterADXProject(LPCWSTR szRuntimeVersion, LPCWSTR szAssemblyName, LPCWSTR szClassName, BOOL mode)
{
    VARIANT v;
    HRESULT hr = S_OK;
    DISPPARAMS dispparams;
    DISPID dispid = NULL;
    _ObjectHandle *pObjHandle = NULL;
    CComBSTR bstrMember(L"UNMANAGEDCALLTOREGISTERCLASS");
 
    IfFailGo(LoadCLR(szRuntimeVersion));
    IfFailGo(CreateLocalAppDomain());
    IfFailGo(m_pLocalDomain->CreateInstance(CComBSTR(szAssemblyName), CComBSTR(szClassName), &pObjHandle));
    VariantInit(&v);
    IfFailGo(pObjHandle->Unwrap(&v));
    IfFailGo(v.pdispVal->GetIDsOfNames(IID_NULL, &bstrMember, 1, LOCALE_SYSTEM_DEFAULT, &dispid));
    memset(&dispparams, 0, sizeof dispparams);
    dispparams.cArgs = 1;
    dispparams.cNamedArgs = 0;
    dispparams.rgvarg = new VARIANT[dispparams.cArgs];
    dispparams.rgvarg[0].boolVal = mode;
    dispparams.rgvarg[0].vt = VT_BOOL;
    IfFailGo(v.pdispVal->Invoke(dispid, IID_NULL, 0, DISPATCH_METHOD, &dispparams, NULL, NULL, NULL));
 
Error:
    return hr;
}
 
static HRESULT GetDllDirectory(TCHAR *szPath, DWORD nPathBufferSize)
{
    HMODULE hInstance = _AtlBaseModule.GetModuleInstance();
 
    if (hInstance == 0)
    {
        return E_FAIL;
    }
 
    TCHAR szModule[MAX_PATH + 1];
    DWORD dwFLen = ::GetModuleFileName(hInstance, szModule, MAX_PATH);
 
    if (dwFLen == 0)
    {
        return E_FAIL;
    }
 
    TCHAR *pszFileName;
    dwFLen = ::GetFullPathName(szModule, nPathBufferSize, szPath, &pszFileName);
    if (dwFLen == 0 || dwFLen >= nPathBufferSize)
    {
        return E_FAIL;
    }
 
    *pszFileName = 0;
    return S_OK;
}

